<?php


namespace Drupal\openid_claveunica\Form;


use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\openid_claveunica\AccountProfileCheckerInterface;
use Drupal\user\Entity\User;
use Drupal\user\UserInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Class CompleteProfileForm
 *
 * @package Drupal\openid_claveunica\Form
 */
class CompleteProfileForm extends FormBase {

  /**
   * @var \Drupal\openid_claveunica\AccountProfileCheckerInterface
   */
  protected $accountProfileChecker;

  /**
   * ClaveUnicaController constructor.
   *
   * @param \Drupal\openid_claveunica\AccountProfileCheckerInterface $account_profile_checker
   */
  public function __construct(AccountProfileCheckerInterface $account_profile_checker) {
    $this->accountProfileChecker = $account_profile_checker;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): CompleteProfileForm {
    return new static(
      $container->get('openid_claveunica.account_profile_checker')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'openid_claveunica_complete_login_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, UserInterface $user = NULL, $hash = NULL) {
    $form['helptext'] = array(
      '#markup' => $this->t('Please fill this form to complete your registration.'),
    );

    $form['mail'] = array(
      '#type' => 'email',
      '#title' => $this->t('E-mail address'),
      '#description' => $this->t('A valid e-mail address. All e-mails from the system will be sent to this address. The e-mail address is not made public and will only be used if you wish to receive a new password or wish to receive certain news or notifications by e-mail.'),
      '#required' => TRUE,
      '#default_value' => '',
    );

    $form['uid'] = array(
      '#type' => 'value',
      '#value' => $user->id(),
    );
    $form['id_hash'] = array(
      '#type' => 'value',
      '#value' => $hash,
    );
    $form['actions'] = array(
      '#type' => 'actions',
    );
    $form['actions']['save'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Complete'),
    );

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $uid = $form_state->getValue('uid');
    $account = User::load($uid);
    $id_hash = $form_state->getValue('id_hash');
    if (!$this->accountProfileChecker->checkHashCompleteProfile($account, $id_hash)) {
      $this->messenger()->addError($this->t('User ID cannot be identified.'));
      $response = new RedirectResponse(Url::fromRoute('<front>')->toString());
      $response->send();
    }

    // Trim whitespace from mail, to prevent confusing 'e-mail not valid'
    // warnings often caused by cutting and pasting.
    $mail = trim($form_state->getValue('mail'));
    $form_state->setValue($form['mail'], $mail);

    $account->setEmail($mail);
    $violations = $account->validate();

    foreach ($violations->getEntityViolations() as $violation) {
      /** @var \Symfony\Component\Validator\ConstraintViolationInterface $violation */
      $form_state->setErrorByName(str_replace('.', '][', $violation->getPropertyPath()), $violation->getMessage());
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $account = User::load($form_state->getValue('uid'));
    $account->setEmail($form_state->getValue('mail'));
    $account->save();
    $account->setLastLoginTime(REQUEST_TIME);

    \Drupal::currentUser()->setAccount($account);

    \Drupal::entityTypeManager()->getStorage('user')->updateLastLoginTimestamp($account);
    \Drupal::logger('openid_claveunica')->info('%name filled missing fields.', array('%name' => $account->getAccountName()));

    \Drupal::service('session')->migrate();
    \Drupal::service('session')->set('uid', $account->id());
    \Drupal::moduleHandler()->invokeAll('user_login', [$account]);

    $form_state->setRedirectUrl(Url::fromRoute('<front>'));
  }

}
